@@ -5,6 +5,7 @@ use warnings;
 use base qw(
     DateTime::Format::Natural::Calc
     DateTime::Format::Natural::Duration
+    DateTime::Format::Natural::Expand
     DateTime::Format::Natural::Extract
     DateTime::Format::Natural::Formatted
     DateTime::Format::Natural::Helpers
@@ -14,12 +15,13 @@ use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
+use DateTime::TimeZone ();
 use List::MoreUtils qw(all any none);
 use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-our $VERSION = '0.98';
+our $VERSION = '1.03';
 
 validation_options(
     on_fail => sub
@@ -92,13 +94,19 @@ sub _init_check
             optional => true,
         },
         time_zone => {
-            type => SCALAR,
+            type => SCALAR | OBJECT,
             optional => true,
             callbacks => {
                 'valid timezone' => sub
                 {
-                    eval { DateTime::TimeZone->new(name => shift) };
-                    !$@;
+                    my $val = shift;
+                    if (blessed($val)) {
+                        return $val->isa('DateTime::TimeZone');
+                    }
+                    else {
+                        eval { DateTime::TimeZone->new(name => $val) };
+                        return !$@;
+                    }
                 }
             },
         },
@@ -230,7 +238,7 @@ sub _params_init
         local $_ = ${$params->{string}};
         s/^\s+//;
         s/\s+$//;
-        $_;
+        $_
     };
 }
 
@@ -278,7 +286,7 @@ sub parse_datetime_duration
     $self->_params_init(@_, { string => \$duration_string });
     my $timespan_sep = $self->{data}->__timespan('literal');
 
-    my @date_strings = $duration_string =~ /\b $timespan_sep \b/ix
+    my @date_strings = $duration_string =~ /\s+ $timespan_sep \s+/ix
       ? do { $self->{duration} = true;
              split /\s+ $timespan_sep \s+/ix, $duration_string }
       : do { $self->{duration} = false;
@@ -371,15 +379,24 @@ sub _process
 
     if (!exists $self->{lookup}) {
         foreach my $keyword (keys %{$self->{data}->__grammar('')}) {
-            push @{$self->{lookup}{scalar @{$self->{data}->__grammar($keyword)->[0]}}}, $keyword;
+            my $count = scalar @{$self->{data}->__grammar($keyword)->[0]};
+            push @{$self->{lookup}{$count}}, [ $keyword, false ];
+            if ($self->_expand_for($keyword)) {
+                push @{$self->{lookup}{$count + 1}}, [ $keyword, true ];
+            }
         }
     }
 
-    PARSE: foreach my $keyword (@{$self->{lookup}{$self->{count}{tokens}} || []}) {
-        my @grammar = @{$self->{data}->__grammar($keyword)};
-        my $types = shift @grammar;
+    PARSE: foreach my $lookup (@{$self->{lookup}{$self->{count}{tokens}} || []}) {
+        my ($keyword, $expandable) = @$lookup;
 
-        foreach my $expression (@grammar) {
+        my @grammar = @{$self->{data}->__grammar($keyword)};
+        my $types_entry = shift @grammar;
+
+        @grammar = $self->_expand($keyword, $types_entry, \@grammar) if $expandable;
+
+        foreach my $entry (@grammar) {
+            my ($types, $expression) = $expandable ? @$entry : ($types_entry, $entry);
             my $valid_expression = true;
             my $definition = $expression->[0];
             my @positions = sort {$a <=> $b} keys %$definition;
@@ -413,7 +430,7 @@ sub _process
                 }
             }
             if ($valid_expression && @{$expression->[2]}) {
-                my $i;
+                my $i = 0;
                 foreach my $check (@{$expression->[2]}) {
                     my @pos = @{$expression->[1][$i++]};
                     my $error;
@@ -426,6 +443,7 @@ sub _process
             }
             if ($valid_expression) {
                 $self->_set_valid_exp;
+                my @truncate_to = @{$expression->[6]->{truncate_to} || []};
                 my $i = 0;
                 foreach my $positions (@{$expression->[3]}) {
                     my ($c, @values);
@@ -441,6 +459,7 @@ sub _process
                     }
                     my $worker = "SUPER::$expression->[5]->[$i]";
                     $self->$worker(@values, $expression->[4]->[$i++]);
+                    $self->_truncate(shift @truncate_to);
                 }
                 %opts = %{$expression->[6]};
                 $self->{keyword} = $keyword;
@@ -452,24 +471,32 @@ sub _process
     $self->_post_process(%opts);
 }
 
+sub _truncate
+{
+    my $self = shift;
+    my ($truncate_to) = @_;
+
+    return unless defined $truncate_to;
+
+    my @truncate_to = map { $_ =~ /_/ ? split /_/, $_ : $_ } $truncate_to;
+    my $i = 0;
+    my @units = @{$self->{data}->__units('ordered')};
+    my %indexes = map { $_ => $i++ } @units;
+    foreach my $unit (@truncate_to) {
+        my $index = $indexes{$unit} - 1;
+        if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
+            $self->{datetime}->truncate(to => $unit);
+            last;
+        }
+    }
+}
+
 sub _post_process
 {
     my $self = shift;
     my %opts = @_;
 
-    if (exists $opts{truncate_to}) {
-        my @truncate_to = ref $opts{truncate_to} eq 'ARRAY' ? @{$opts{truncate_to}} : ($opts{truncate_to});
-        my $i = 0;
-        my @units = @{$self->{data}->__units('ordered')};
-        my %indexes = map { $_ => $i++ } @units;
-        foreach my $unit (@truncate_to) {
-            my $index = $indexes{$unit} - 1;
-            if (defined $units[$index] && !exists $self->{modified}{$units[$index]}) {
-                $self->{datetime}->truncate(to => $unit);
-                last;
-            }
-        }
-    }
+    delete $opts{truncate_to};
 
     if ($self->{Prefer_future} &&
         (exists $opts{prefer_future} && $opts{prefer_future})
@@ -587,9 +614,11 @@ sub _set_datetime
 1;
 __END__
 
+=encoding ISO8859-1
+
 =head1 NAME
 
-DateTime::Format::Natural - Create machine readable date/time with natural parsing logic
+DateTime::Format::Natural - Parse informal natural language date/time strings
 
 =head1 SYNOPSIS
 
@@ -597,30 +626,43 @@ DateTime::Format::Natural - Create machine readable date/time with natural parsi
 
  $parser = DateTime::Format::Natural->new;
 
- $date_string  = $parser->extract_datetime($extract_string);
- @date_strings = $parser->extract_datetime($extract_string);
-
  $dt = $parser->parse_datetime($date_string);
  @dt = $parser->parse_datetime_duration($date_string);
 
+ $date_string  = $parser->extract_datetime($extract_string);
+ @date_strings = $parser->extract_datetime($extract_string);
+
  if ($parser->success) {
      # operate on $dt/@dt, for example:
-     printf("%02d.%02d.%4d %02d:%02d:%02d\n", $dt->day,
-                                              $dt->month,
-                                              $dt->year,
-                                              $dt->hour,
-                                              $dt->min,
-                                              $dt->sec);
+     print $dt->strftime('%d.%m.%Y %H:%M:%S'), "\n";
  } else {
      warn $parser->error;
  }
 
  @traces = $parser->trace;
 
+ # examples
+
+ 12:14 PM
+ next tuesday at 2am
+ tomorrow morning
+ 4pm yesterday
+ 10 weeks ago
+
+ 1st tuesday last november
+ 2nd friday in august
+ final thursday in april
+
+ for 3 hours
+ monday to friday
+ 1 April 10 am to 1 May 8am
+
+ jan 24, 2011 12:00
+
 =head1 DESCRIPTION
 
-C<DateTime::Format::Natural> takes a string with a human readable date/time and creates a
-machine readable one by applying natural parsing logic.
+C<DateTime::Format::Natural> parses informal natural language date/time strings.
+In addition, parsable date/time substrings may be extracted from ordinary strings.
 
 =head1 CONSTRUCTOR
 
@@ -677,7 +719,7 @@ which may be selectively changed.
 
 =head2 parse_datetime
 
-Returns a L<DateTime> object constructed from a human readable date/time string.
+Returns a L<DateTime> object constructed from a natural language date/time string.
 
  $dt = $parser->parse_datetime($date_string);
  $dt = $parser->parse_datetime(string => $date_string);
@@ -692,7 +734,7 @@ The date string.
 
 =head2 parse_datetime_duration
 
-Returns one or two L<DateTime> objects constructed from a human readable
+Returns one or two L<DateTime> objects constructed from a natural language
 date/time string which may contain timespans/durations. I<Same> interface
 and options as C<parse_datetime()>, but should be explicitly called in
 list context.
@@ -735,13 +777,13 @@ one string is commonly returned for durations. Useful as a debugging aid.
 The grammar handling has been rewritten to be easily extendable and hence
 everybody is encouraged to propose sensible new additions and/or changes.
 
-See the classes C<DateTime::Format::Natural::Lang::[language_code]> if
-you're intending to hack a bit on the grammar guts.
+See the class L<DateTime::Format::Natural::Lang::EN> if you're intending
+to hack a bit on the grammar guts.
 
 =head1 EXAMPLES
 
-See the classes C<DateTime::Format::Natural::Lang::[language_code]> for an
-overview of currently valid input.
+See the class L<DateTime::Format::Natural::Lang::EN> for an overview of
+currently valid input.
 
 =head1 BUGS & CAVEATS
 
@@ -795,10 +837,14 @@ valuable suggestions and patches:
  Colm Dougan
  Chifung Fan
  Xiao Yafeng
+ Roman Filippov
+ David Steinbrunner
+ Debian Perl Group
+ Tim Bunce
 
 =head1 SEE ALSO
 
-L<DateTime>, L<Date::Calc>, L<http://datetime.perl.org>
+L<dateparse>, L<DateTime>, L<Date::Calc>, L<http://datetime.perl.org>
 
 =head1 AUTHOR
 
